/***************************************************************************
 *
 * client.c - Types and definitions for the SpaceDuel3D client
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef CLIENT_H
#define CLIENT_H

#include <vector.h>
#include <transfer.h>

/* Field-of-view in degrees */

#define SD_FOV                60.0

/* Front and back clip plane distances */

#define SD_FRONT_CLIP         4.0
#define SD_BACK_CLIP          1500.0


/* Number of animation frames for death and warning messages */

#define     SD_DEATHFRAMES    10
#define     SD_WARNINGFRAMES  10


/* Grouping of control keys */

struct controlKeys
{
  unsigned char     thrustIncr;
  unsigned char     thrustDecr;
  unsigned char     headingIncr;
  unsigned char     headingDecr;
  unsigned char     pitchIncr;
  unsigned char     pitchDecr;
  unsigned char     rollIncr;
  unsigned char     rollDecr;
  unsigned char     fire;
};
typedef struct controlKeys        controlKeys;


/* Display options */

struct displayOps
{
  int               wantTextures;
  int               wantAsteroidLighting;
  int               wantDithering;
  int               wantTexturedStarfield;
  vector            globalAmbientFactor;
};
typedef struct displayOps     displayOps;

extern displayOps displayOptions;


/*
 * matchDOF
 *
 * Input:
 *    heading
 *       Direction of flight vector
 *    up
 *       Up vector
 *
 * Description:
 *    Creates a rotation matrix that maps the standard X, Y, Z
 *    coordinate system onto the right-hand coordinate system
 *    defined by the heading and up vectors. It also multiplies
 *    the current OpenGL transformation matrix with the rotation
 *    matrix.
 *
 */

void matchDOF (vector heading, vector up);


/*
 * getWinDimensions
 *
 * Input:
 *    width
 *       Pointer to float variable where width is filled in.
 *    height
 *       Pointer to float variable where height is filled in.
 * 
 * Description:
 *    Fills in the current dimensions of the screen window in
 *    the specified variables.
 *
 */

void getWinDimensions (float *width, float *height);


/*
 * resetOpenGL
 *
 * Description:
 *    Reset OpenGL parameters to minimal (= max performance).
 *
 */

void resetOpenGL (void);


/*
 * nearEnough
 *
 * Input:
 *    p
 *       Point.
 *    eye
 *       Pointer to eyepoint data.
 *
 * Description:
 *    Checks if a point is close enough to the camera to be drawn.
 *
 * Returns:
 *    true if point should be drawn.
 *    false otherwise.
 *
 */

int nearEnough (vector p, eyePointXferData *eye);


/*
 * pointVisible
 *
 * Input:
 *    p
 *       Point.
 *    eye
 *       Pointer to eyepoint data.
 *
 * Description:
 *    Checks if a point is visible to the camera.
 *
 * Returns:
 *    true if point should be drawn.
 *    false otherwise.
 *
 */

int pointVisible (vector p, eyePointXferData *eye);

#endif /* CLIENT_H */
