/***************************************************************************
 *
 * readppm.h - read ASCII .PPM files and store them as texture data.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef READPPM_H
#define READPPM_H


/*
 * Function that creates a texel from RGB data. The function
 * must provide R, G, B and A values in the interval [0, 1] and
 * store them in memory locations texel, texel+1, texel+2 and texel+3.
 *
 */

typedef void (*createTexelFunc) (float    *texel, 
				 int       R,
				 int       G,
				 int       B,
				 int       maxColorVal);


/*
 * readPPM
 *
 * Input:
 *    filename
 *       Name of ASCII .PPM file.
 *    func
 *       Pointer to function that creates texels from RGB data.
 *    w
 *       Pointer to integer where image width is stored.
 *    h
 *       Pointer to integer where image height is stored.
 *
 * Description: 
 *    Reads ASCII PPM data from the specified file. Creates a texture and
 *    stores the image dimensions in memory pointed to by w and h.
 *
 * Returns:
 *    Pointer to texture data if successful. NULL otherwise.
 *
 */

float *readPPM (char             *filename, 
		createTexelFunc   func,
		int              *w,
		int              *h);


/*
 * readBinaryPPM
 *
 * Input:
 *    filename
 *       Name of binary/raw .PPM file.
 *    func
 *       Pointer to function that creates texels from RGB data.
 *    w
 *       Pointer to integer where image width is stored.
 *    h
 *       Pointer to integer where image height is stored.
 *
 * Description: 
 *    Reads binary PPM data from the specified file. Creates a texture and
 *    stores the image dimensions in memory pointed to by w and h.
 *
 * Returns:
 *    Pointer to texture data if successful. NULL otherwise.
 *
 */

float *
readBinaryPPM (char              *filename,
	       createTexelFunc    func,
	       int               *w,
	       int               *h);


#endif   /* READPPM_H */
