/***************************************************************************
 *
 * s_asteroid.h - Server side utility functions for asteroid entity.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef S_ASTEROID_H
#define S_ASTEROID_H

#include <s_world.h>


/*
 * Space-partitioning parameters
 *
 */

/* Number of partitions along X, Y and Z axes */

#define SD_ASTEROIDS_X       3
#define SD_ASTEROIDS_Y       3
#define SD_ASTEROIDS_Z       3

#define SD_ASTEROIDS (SD_ASTEROIDS_X * SD_ASTEROIDS_Y * SD_ASTEROIDS_Z)

/* Length of partition box side */

#define SD_ASTEROID_BOX_SIZE     500.0


/*
 * Asteroid parameters
 *
 */

#define SD_ASTEROID_RADIUS       100.0

#define SD_ASTEROID_MAXSPEED     5.0    /* Degrees / second */


/*
 * Asteroid
 *
 */

struct asteroidData
{
  vector            position;
  float             radius;
  float             angleX;
  float             angleY;
  float             angleZ;

  float             angleSpeedX;    /* Rotation speed in degrees / sec */
  float             angleSpeedY;
  float             angleSpeedZ;
};
typedef struct asteroidData        asteroidData;



/*
 * createAsteroids
 *
 * Description:
 *    Creates asteroid world entries.
 *
 */

void createAsteroids (void);


/*
 * asteroidUpdateFunc
 *
 * Input:
 *    entry
 *       Pointer to (asteroid) world entry.
 *    timeslice
 *       Simulation timeslice in seconds.
 *
 * Description:
 *    Runs asteroid simulation "timeslice" seconds.
 *
 */

void asteroidUpdateFunc (worldEntry *entry, double timeslice);


/*
 * getAsteroidXferData
 *
 * Input:
 *    data
 *       Pointer to transfer data to be filled in.
 *    entry
 *       Pointer to (asteroid) world entry.
 *
 * Description:
 *    Fills in transfer data for the asteroid pointed to by "entry".
 *
 */

void getAsteroidXferData (xferData *data, worldEntry *entry);


/*
 * asteroidCollisionFunc
 *
 * Input:
 *    me
 *       Pointer to (asteroid) world entry.
 *    other
 *       Pointer to a world entry.
 *
 * Description:
 *    The asteroid entity pointed to by "me" has collided with the
 *    entity pointed to by "other". The function takes the appropriate
 *    action depending on the type of "other".
 *
 */

void asteroidCollisionFunc (worldEntry *me, worldEntry *other);


/*
 * outsideAsteroidField
 *
 * Input:
 *    p
 *       Point.
 *
 * Returns:
 *    true if p is outside the asteroid field, false otherwise.
 *
 */

int outsideAsteroidField (vector p);


#endif /* S_ASTEROID_H */
