/***************************************************************************
 *
 * s_client.c - Server side abstraction of a connected client.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef S_CLIENT_H
#define S_CLIENT_H

#include <linkedlist.h>
#include <communication.h>
#include <s_world.h>


/* Maximum number of connected clients */

#define SD_MAXCLIENTS    256



/*
 * Client connection data
 *
 */

struct clientData;
struct worldEntry;

struct clientConnectionData
{
  comChannel          stream;
  comChannel          dgram;

  struct worldEntry  *myData;
};
typedef struct clientConnectionData     clientConnectionData;



/*
 * newClientEntry
 *
 * Input:
 *    stream
 *       Pointer to a stream communication channel.
 *    dgram
 *       Pointer to a datagram communication channel.
 *
 * Description:
 *    Adds a new entry to the client connection database.
 *
 * Returns:
 *    Pointer to the new entry if successful.
 *    NULL if unsuccessful. This indicates that the client
 *    connection database is full.
 *
 */

clientConnectionData *newClientEntry (comChannel     *stream,
				      comChannel     *dgram);


/*
 * newClient
 *
 * Input:
 *    stream
 *       Pointer to stream communication channel.
 *    dgram
 *       Pointer to datagram communication channel.
 *
 * Description:
 *    Creates a new world entry that corresponds to a client.
 *
 */

void newClient (comChannel    *stream,
		comChannel    *dgram);


/*
 * getNumberOfClientConnections
 *
 * Description:
 *    Returns the total number of client connections in the
 *    client connection database.
 *
 */

int getNumberOfClientConnections (void);


/*
 * deleteConnectionData
 *
 * Input:
 *    client
 *       Pointer to client connection data.
 *
 * Description:
 *    Removes the client connection data from the connection
 *    database.
 *
 */

void deleteConnectionData (clientConnectionData   *client);


/*
 * getClientList
 *
 * Returns:
 *    Pointer to the list of clients.
 *
 * NOTE:
 *    This function breaks the abstraction but makes it easier to
 *    iterate through the list of clients. A future version may
 *    return a list iterator instead.
 *
 */

linkedlist *getClientList (void);


#endif /* S_CLIENT_H */
