/***************************************************************************
 *
 * timeutils.h - Time measurning utilities.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef TIMEUTILS_H
#define TIMEUTILS_H


/* Conversion constants between seconds, milliseconds and microseconds */

#define TIME_MSEC_TO_SEC      0.001
#define TIME_SEC_TO_MSEC      1000.0
#define TIME_USEC_TO_SEC      0.000001
#define TIME_SEC_TO_USEC      1000000.0
#define TIME_MSEC_TO_USEC     1000.0
#define TIME_USEC_TO_MSEC     0.001


/*
 * Timestamp.
 *
 */

typedef double timestamp;


/*
 * timeSleepUsec
 *
 * Input:
 *    usec
 *       Number of microseconds.
 *
 * Description:
 *    Puts the current process to sleep for the specified number of
 *    microseconds.
 *
 */

void timeSleepUsec (int usec);


/*
 * timeSleepMsec
 *
 * Input:
 *    msec
 *       Number of milliseconds.
 *
 * Description:
 *    Puts the current process to sleep for the specified number of
 *    milliseconds.
 *
 */

void timeSleepMsec (int msec);



/*
 * getTimeStamp
 *
 * Returns:
 *    A time stamp.
 *
 */

timestamp getTimestamp (void);


/*
 * timeDifference
 *
 * Input:
 *   t
 *      Time stamp.
 *
 * Returns:
 *    Number of seconds between t and now.
 *
 */

double timeDifference (timestamp t);


#endif /* TIMEUTILS_H */
