/***************************************************************************
 *
 * transfer.h - Data exchanged by server and client.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#ifndef TRANSFER_H
#define TRANSFER_H

#include <vector.h>
#include <communication.h>


/*
 * Length of transfer data, i.e. number of entries of type
 * xferData that is sent
 *
 */

#define SD_DATALENGTH 256


/*
 * Handshaking string length
 *
 */

#define SD_HANDSHAKESIZE    80


/*
 * Transfer data size
 *
 */

#define SD_XFERDATASIZE 64


/*
 * Transfer data types
 *
 */

#define SD_EYEPOINT    1
#define SD_VESSEL      2
#define SD_ASTEROID    3
#define SD_BLAST       4
#define SD_EXPLOSION   5


/*
 * End-of-data constant
 *
 */

#define SD_EOD     -1


/*
 * Message types
 *
 */

#define SD_MSG_THRUST         1
#define SD_MSG_HEADING        2
#define SD_MSG_PITCH          3
#define SD_MSG_ROLL           4
#define SD_MSG_FIRE           5

#define SD_MSG_DEATH          6
#define SD_MSG_SCORE          7
#define SD_MSG_TEXT           8
#define SD_MSG_WARNING        9


/*
 * Message data size
 *
 */

#define SD_MESSAGEDATASIZE   256



/*
 * Stream channel message
 *
 */

struct message
{
  int    type;
  char   data[SD_MESSAGEDATASIZE];
};
typedef struct message      message;



/*
 * Transfer data
 *
 */

struct xferData
{
  char      type;
  char      model;
  char      data[SD_XFERDATASIZE];
};
typedef struct xferData       xferData;



/* 
 * Vessel
 *
 */

struct vesselXferData
{
  vector      position;
  vector      heading;
  vector      up;
  char        animFrame;
  float       speed;
};
typedef struct vesselXferData        vesselXferData;


/*
 * Eye point
 *
 */

struct eyePointXferData
{
  vector     position;
  vector     heading;
  vector     up;
  float      speed;
};
typedef struct eyePointXferData      eyePointXferData;


/*
 * Asteroid
 *
 */

struct asteroidXferData
{
  vector      position;
  float       radius;
  float       angleX;
  float       angleY;
  float       angleZ;
};
typedef struct asteroidXferData        asteroidXferData;


/*
 * Blast
 *
 */

struct blastXferData
{
  vector      position;
  vector      heading;
  vector      up;
  char        animFrame;
};
typedef struct blastXferData        blastXferData;


/*
 * Explosion
 *
 */

struct explosionXferData
{
  vector       position;
  char         animFrame;
};
typedef struct explosionXferData      explosionXferData;


/*
 * sendHandshake
 *
 * Input:
 *    channel
 *       Communication channel.
 *
 * Description:
 *    Sends the local handshake string through the channel.
 *
 * Returns:
 *    SD_OK if successful.
 *    SD_ERROR if unsuccessful.
 *
 */

int sendHandshake (comChannel *channel);


/*
 * readHandshake
 *
 * Input:
 *    channel
 *       Communication channel.
 *
 * Description:
 *    Reads the handshake string from the channel and compares it
 *    to the local handshake string.
 *
 * Returns:
 *    SD_OK if successful and the strings match.
 *    SD_ERROR if unsuccessful or the strings doesn't match.
 *
 */

int readHandshake (comChannel *channel);


#endif /* TRANSFER_H */
