/***************************************************************************
 *
 * c_explosion.c - Client side functions that implements an explosion.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <GL/glut.h>

#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <stdio.h>

#include <randomutils.h>

#include <client.h>
#include <c_explosion.h>


particleData     particles[NUM_PARTICLES];    /* holds particle data */
debrisData       debris[NUM_DEBRIS];          /* holds debris data */


/*
 * newSpeed
 *
 * Input:
 *    dest
 *       Vector to be filled in.
 *
 * Description:
 *    Randomize a new speed vector.
 *
 */

void
newSpeed (float dest[3])
{
  float    x;
  float    y;
  float    z;

  x = randomBetweenMinusOneAndOne () * 8.0;
  y = randomBetweenMinusOneAndOne () * 8.0;
  z = randomBetweenMinusOneAndOne () * 8.0;

  dest[0] = x;
  dest[1] = y;
  dest[2] = z;
}


/*
 * createExplosion
 *
 * Description:
 *    Initialize explosion particle system.
 *
 */

void
createExplosion (void)
{
  int              i;

  srand (time (NULL));

  /* Create the particles */

  for (i = 0; i < NUM_PARTICLES; i++)
    {
      particles[i].position[0] = 0.0;
      particles[i].position[1] = 0.0;
      particles[i].position[2] = 0.0;

      particles[i].color[0] = 1.0;
      particles[i].color[1] = 1.0;
      particles[i].color[2] = 0.5;

      newSpeed (particles[i].speed);
    }

  /* Create debris */

  for (i = 0; i < NUM_DEBRIS; i++)
    {
      debris[i].position[0] = 0.0;
      debris[i].position[1] = 0.0;
      debris[i].position[2] = 0.0;

      debris[i].orientation[0] = 0.0;
      debris[i].orientation[1] = 0.0;
      debris[i].orientation[2] = 0.0;

      debris[i].color[0] = 0.7;
      debris[i].color[1] = 0.7;
      debris[i].color[2] = 0.7;

      debris[i].scale[0] = randomBetweenMinusOneAndOne () * 8.0;
      debris[i].scale[1] = randomBetweenMinusOneAndOne () * 8.0;
      debris[i].scale[2] = randomBetweenMinusOneAndOne () * 8.0;

      newSpeed (debris[i].speed);
      newSpeed (debris[i].orientationSpeed);
    }
}


/*
 * drawExplosion
 *
 * Input:
 *    data
 *       Pointer to transfer data.
 *
 * Description:
 *    Draws an explosion based on the transferred data.
 *
 */

void
drawExplosion (xferData           *data,
	       eyePointXferData   *eye)
{
  explosionXferData     expl;
  vector                p;
  vector                c;
  vector                o;
  int                   i;

  memcpy ((char *) &expl, (char *) &data->data, sizeof (expl));

  if (nearEnough (expl.position, eye))
    {
      if (pointVisible (expl.position, eye))
	{
	  glPushMatrix ();
	  glTranslatef (expl.position[0],
			expl.position[1],
			expl.position[2]);

	  /* Draw particles */
	  
	  glBegin (GL_POINTS);
	  for (i = 0; i < NUM_PARTICLES; i++)
	    {
	      p[0] = (float) expl.animFrame * (particles[i].speed[0] * 0.2);
	      p[1] = (float) expl.animFrame * (particles[i].speed[1] * 0.2);
	      p[2] = (float) expl.animFrame * (particles[i].speed[2] * 0.2);
	      
	      c[0] = particles[i].color[0] - (1.0 / ((float) expl.animFrame));
	      c[1] = particles[i].color[1] - (1.0 / ((float) expl.animFrame));
	      c[2] = particles[i].color[2] - (1.0 / ((float) expl.animFrame));

	      glColor3fv (c);
	      glVertex3fv (p);
	    }
	  glEnd ();

	  /* Draw debris */

	  glNormal3f (0.0, 0.0, 1.0);
	  glEnable (GL_LIGHTING);
	  glEnable (GL_NORMALIZE);

	  for (i = 0; i < NUM_DEBRIS; i++)
	    {
	      p[0] = (float) expl.animFrame * (debris[i].speed[0] * 0.1);
	      p[1] = (float) expl.animFrame * (debris[i].speed[1] * 0.1);
	      p[2] = (float) expl.animFrame * (debris[i].speed[2] * 0.1);
	      
	      o[0] = (float) expl.animFrame 
		* (debris[i].orientationSpeed[0] * 10);
	      o[1] = (float) expl.animFrame
		* (debris[i].orientationSpeed[1] * 10);
	      o[2] = (float) expl.animFrame
		* (debris[i].orientationSpeed[2] * 10);

	      glColor3fv (debris[i].color);

	      glPushMatrix ();
      
	      glTranslatef (p[0], p[1], p[2]);
      
	      glRotatef (o[0], 1.0, 0.0, 0.0);
	      glRotatef (o[1], 0.0, 1.0, 0.0);
	      glRotatef (o[2], 0.0, 0.0, 1.0);
      
	      glScalef (debris[i].scale[0],
			debris[i].scale[1],
			debris[i].scale[2]);

	      glBegin (GL_TRIANGLES);
	      glVertex3f (0.0, 0.5, 0.0);
	      glVertex3f (-0.25, 0.0, 0.0);
	      glVertex3f (0.25, 0.0, 0.0);
	      glEnd ();	  
	  
	      glPopMatrix ();
	    }  

	  glPopMatrix ();
	}
    }
}
