/***************************************************************************
 *
 * c_sphere.c - Function that draws a sphere with texture coordinates.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <math.h>
#include <GL/glut.h>

#include <general.h>
#include <vector.h>

void
sphereCoord (vector    p,
	     GLfloat  *u,
	     GLfloat  *v,
	     int       i,
	     int       j,
	     int       patchesU,
	     int       patchesV)
{
  GLfloat                   theta;
  GLfloat                   phi;

  *u = (GLfloat) i / patchesU;
  *v = 1.0 - ((GLfloat) j / patchesV);

  theta = M_PI * ((GLfloat) j / patchesV);
  phi = 2 * M_PI * ((GLfloat) i / patchesU);

  vecSpherical (p, theta, phi);
}


/* Generate the top */

void
drawSphereTop (float   radius,
	       int     patchesU,
	       int     patchesV)
{
  int           i;
  vector        p;
  GLfloat       u;
  GLfloat       v;

  glBegin (GL_TRIANGLE_FAN);
  
  glNormal3f (0.0, 0.0, 1.0);
  glTexCoord2f (0.5, 1.0);
  glVertex3f (0.0, 0.0, radius);

  for (i = 0; i <= patchesU; i++)
    {
      sphereCoord (p, &u, &v, i, 1, patchesU, patchesV);
      glNormal3fv (p);
      glTexCoord2f (u, v);
      vecMult (p, radius, p);
      glVertex3fv (p);
    }

  glEnd ();
}


void
drawSphereBottom (float    radius,
		  int      patchesU,
		  int      patchesV)
{
  int           i;
  vector        p;
  GLfloat       u;
  GLfloat       v;

  glBegin (GL_TRIANGLE_FAN);
  
  glNormal3f (0.0, 0.0, -1.0);
  glTexCoord2f (0.5, 0.0);
  glVertex3f (0.0, 0.0, -radius);

  for (i = 0; i <= patchesU; i++)
    {
      sphereCoord (p,
		   &u,
		   &v,
		   patchesU - i,
		   patchesV - 1,
		   patchesU,
		   patchesV);
      glNormal3fv (p);
      glTexCoord2f (u, v);
      vecMult (p, radius, p);
      glVertex3fv (p);
    }

  glEnd ();
}


void
drawSphereMiddle (float      radius,
		  int        patchesU,
		  int        patchesV)
{
  int           i;
  int           j;
  vector        p;
  GLfloat       u;
  GLfloat       v;

  glBegin (GL_QUAD_STRIP);

  for (j = 2; j < patchesV; j++)
    {
      for (i = 0; i <= patchesU; i++)
	{
	  sphereCoord (p,
		       &u,
		       &v,
		       i,
		       j - 1, 
		       patchesU, 
		       patchesV);
	  glNormal3fv (p);
	  glTexCoord2f (u, v);
	  vecMult (p, radius, p);
	  glVertex3fv (p);
	  sphereCoord (p,
		       &u,
		       &v,
		       i,
		       j,
		       patchesU, 
		       patchesV);
	  glNormal3fv (p);
	  glTexCoord2f (u, v);
	  vecMult (p, radius, p);
	  glVertex3fv (p);
	}
    }

  glEnd ();
}


/*
 * drawSphere
 *
 * Input:
 *    radius
 *       Sphere radius.
 *    patchesU
 *       Number of patches in U direction.
 *    patchesV
 *       Number of patches in V direction.
 *
 * Description:
 *    Draws a sphere with texture coordinates. Can replace
 *    GLUTs glutSolidSphere.
 *
 */

void
drawSphere (float       radius,
	    int         patchesU,
	    int         patchesV)
{
  drawSphereTop (radius, patchesU, patchesV);
  drawSphereMiddle (radius, patchesU, patchesV);
  drawSphereBottom (radius, patchesU, patchesV);
}
