/***************************************************************************
 *
 * c_vessel.c - Client-side functions for drawing a vessel.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <GL/glut.h>
#include <string.h>
#include <math.h>

#include <client.h>
#include <c_vessel.h>

GLint     vesselDL;   /* holds the name of the vessel display list */


/*
 * createVesselDL
 *
 * Description:
 *    Creates the vessel display list.
 *
 */

void
createVesselDL (GLint      dl)
{
  GLfloat    amb[4] = { 0.2, 0.2, 0.2, 1.0 };
  GLfloat    dif[4] = { 0.4, 0.4, 0.45, 1.0 };
  GLfloat    spc[4] = { 0.22, 0.2, 0.25, 1.0 };

  vesselDL = glGenLists (1);
  
  glNewList (vesselDL, GL_COMPILE);

  glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, amb);
  glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, dif);
  glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, spc);
  glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 30.0);
  glLightModelf (GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

  glEnable (GL_CULL_FACE);
  glEnable (GL_DEPTH_TEST);
  glEnable (GL_NORMALIZE); 
  glEnable (GL_LIGHTING);
  glShadeModel (GL_FLAT);

  /* Draw the vessel */

  glBegin (GL_TRIANGLES);

  glColor3f (0.5, 0.5, 0.5);

  glNormal3f (-4.0, 4.0, -1.0);
  glVertex3f (-1.0, 0.0, -8.0);
  glVertex3f (-2.0, 0.0, -4.0);
  glVertex3f (-1.0, 1.0, -4.0);

  glNormal3f (4.0, 4.0, -1.0);
  glVertex3f (1.0, 0.0, -8.0);
  glVertex3f (1.0, 1.0, -4.0);
  glVertex3f (2.0, 0.0, -4.0);

  glColor3f (0.4, 0.4, 0.4);
  
  glNormal3f (-4.0, -4.0, -1.0);
  glVertex3f (-1.0, 0.0, -8.0);
  glVertex3f (-1.0, -1.0, -4.0);
  glVertex3f (-2.0, 0.0, -4.0);

  glNormal3f (4.0, -4.0, -1.0);
  glVertex3f (1.0, 0.0, -8.0);
  glVertex3f (2.0, 0.0, -4.0);
  glVertex3f (1.0, -1.0, -4.0);

  glEnd ();

  glBegin (GL_QUADS);

  glColor3f (0.6, 0.6, 0.6);

  glNormal3f (0.0, 8.0, -2.0);
  glVertex3f (-1.0, 0.0, -8.0);
  glVertex3f (-1.0, 1.0, -4.0);
  glVertex3f ( 1.0, 1.0, -4.0);
  glVertex3f ( 1.0, 0.0, -8.0);

  glColor3f (0.3, 0.3, 0.3);

  glNormal3f (0.0, -8.0, -2.0);
  glVertex3f (-1.0,  0.0, -8.0);
  glVertex3f ( 1.0,  0.0, -8.0);
  glVertex3f ( 1.0, -1.0, -4.0);  
  glVertex3f (-1.0, -1.0, -4.0);

  glColor3f (0.5, 0.5, 0.5);

  glNormal3f (-2.0, 2.0, 0.0);
  glVertex3f (-2.0, 0.0, -4.0);
  glVertex3f (-2.0, 0.0, -2.0);
  glVertex3f (-1.0, 1.0, -2.0);
  glVertex3f (-1.0, 1.0, -4.0);

  glNormal3f (2.0, 2.0, 0.0);
  glVertex3f (2.0, 0.0, -4.0);
  glVertex3f (1.0, 1.0, -4.0);
  glVertex3f (1.0, 1.0, -2.0);
  glVertex3f (2.0, 0.0, -2.0);

  glColor3f (0.4, 0.4, 0.4);

  glNormal3f (-2.0, -2.0, 0.0);
  glVertex3f (-2.0,  0.0, -4.0);
  glVertex3f (-1.0, -1.0, -4.0);
  glVertex3f (-1.0, -1.0, -2.0);
  glVertex3f (-2.0,  0.0, -2.0);

  glNormal3f (2.0, -2.0, 0.0);
  glVertex3f (2.0,  0.0, -4.0);
  glVertex3f (2.0,  0.0, -2.0);
  glVertex3f (1.0, -1.0, -2.0);
  glVertex3f (1.0, -1.0, -4.0);

  glColor3f (0.6, 0.6, 0.6);

  glNormal3f (0.0, 1.0, 0.0);
  glVertex3f (-1.0, 1.0, -4.0);
  glVertex3f (-1.0, 1.0, -2.0);
  glVertex3f ( 1.0, 1.0, -2.0);
  glVertex3f ( 1.0, 1.0, -4.0);

  glColor3f (0.3, 0.3, 0.3);

  glNormal3f (0.0, -1.0, 0.0);
  glVertex3f (-1.0, -1.0, -4.0);
  glVertex3f ( 1.0, -1.0, -4.0);
  glVertex3f ( 1.0, -1.0, -2.0);
  glVertex3f (-1.0, -1.0, -2.0);

  glEnd ();

  glBegin (GL_TRIANGLES);

  glColor3f (0.5, 0.5, 0.5);

  glNormal3f (-2.0, 2.0, 0.5);
  glVertex3f (-2.0, 0.0, -2.0);
  glVertex3f (-1.5, 0.0,  0.0);
  glVertex3f (-1.0, 1.0, -2.0);

  /*  glNormal3f (-1.0, 1.0, 0.25); */  /* The same as -2 2 0.5 */
  glVertex3f (-1.0, 1.0, -2.0);
  glVertex3f (-1.5, 0.0,  0.0);
  glVertex3f (-1.0, 0.5,  0.0);

  glNormal3f (2.0, 2.0, 0.5);
  glVertex3f (2.0, 0.0, -2.0);
  glVertex3f (1.0, 1.0, -2.0);
  glVertex3f (1.5, 0.0,  0.0);

  /*  glNormal3f (1.0, 1.0, 0.25); */   /* Same reason */
  glVertex3f (1.0, 1.0, -2.0);
  glVertex3f (1.0, 0.5,  0.0);
  glVertex3f (1.5, 0.0,  0.0);

  glColor3f (0.4, 0.4, 0.4);

  glNormal3f (-2.0, -2.0, 0.5);
  glVertex3f (-2.0,  0.0, -2.0);
  glVertex3f (-1.0, -1.0, -2.0);
  glVertex3f (-1.5,  0.0,  0.0);

  glVertex3f (-1.0, -1.0, -2.0);
  glVertex3f (-1.0, -0.5,  0.0);
  glVertex3f (-1.5,  0.0,  0.0);

  glNormal3f (2.0, -2.0, 0.5);
  glVertex3f (2.0,  0.0, -2.0);
  glVertex3f (1.5,  0.0,  0.0);
  glVertex3f (1.0, -1.0, -2.0);

  glVertex3f (1.0, -1.0, -2.0);
  glVertex3f (1.5,  0.0,  0.0);
  glVertex3f (1.0, -0.5,  0.0);

  glEnd ();

  glBegin (GL_QUADS);

  glColor3f (0.6, 0.6, 0.6);
  
  glNormal3f (0.0, 4.0, 1.0);
  glVertex3f (-1.0, 1.0, -2.0);
  glVertex3f (-1.0, 0.5, 0.0);
  glVertex3f ( 1.0, 0.5, 0.0);
  glVertex3f ( 1.0, 1.0, -2.0);

  glColor3f (0.3, 0.3, 0.3);

  glNormal3f (0.0, -4.0, 1.0);
  glVertex3f (-1.0, -1.0, -2.0);
  glVertex3f ( 1.0, -1.0, -2.0);
  glVertex3f ( 1.0, -0.5, 0.0);
  glVertex3f (-1.0, -0.5, 0.0);

  glEnd ();

  glBegin (GL_QUADS);
  
  glColor3f (0.5, 0.5, 0.5);

  glNormal3f (-4.0, 4.0, 0.0);
  glVertex3f (-1.5, 0.0, 0.0);
  glVertex3f (-1.5, 0.0, 8.0);
  glVertex3f (-1.0, 0.5, 8.0);
  glVertex3f (-1.0, 0.5, 0.0);

  glNormal3f (4.0, 4.0, 0.0);
  glVertex3f (1.5, 0.0, 0.0);
  glVertex3f (1.0, 0.5, 0.0);
  glVertex3f (1.0, 0.5, 8.0);
  glVertex3f (1.5, 0.0, 8.0);

  glColor3f (0.4, 0.4, 0.4);

  glNormal3f (-4.0, -4.0, 0.0);
  glVertex3f (-1.5,  0.0, 0.0);
  glVertex3f (-1.0, -0.5, 0.0);
  glVertex3f (-1.0, -0.5, 8.0);
  glVertex3f (-1.5,  0.0, 8.0);

  glNormal3f (4.0, -4.0, 0.0);
  glVertex3f (1.5,  0.0, 0.0);
  glVertex3f (1.5,  0.0, 8.0);
  glVertex3f (1.0, -0.5, 8.0);
  glVertex3f (1.0, -0.5, 0.0);

  glColor3f (0.6, 0.6, 0.6);

  glNormal3f (0.0, 1.0, 0.0);
  glVertex3f (-1.0, 0.5, 0.0);
  glVertex3f (-1.0, 0.5, 8.0);
  glVertex3f ( 1.0, 0.5, 8.0);
  glVertex3f ( 1.0, 0.5, 0.0);

  glColor3f (0.3, 0.3, 0.3);

  glNormal3f (0.0, -1.0, 0.0);
  glVertex3f (-1.0, -0.5, 0.0);
  glVertex3f ( 1.0, -0.5, 0.0);
  glVertex3f ( 1.0, -0.5, 8.0);
  glVertex3f (-1.0, -0.5, 8.0);

  glEnd ();

  glBegin (GL_POLYGON);

  glColor3f (0.5, 0.5, 0.5);

  glNormal3f (0.0, 0.0, 1.0);
  glVertex3f (-1.5,  0.0, 8.0);
  glVertex3f (-1.5,  0.0, 8.0);
  glVertex3f (-1.0, -0.5, 8.0);
  glVertex3f ( 1.0, -0.5, 8.0);
  glVertex3f ( 1.5,  0.0, 8.0);
  glVertex3f ( 1.0,  0.5, 8.0);
  glVertex3f (-1.0,  0.5, 8.0);

  glEnd ();

  glDisable (GL_CULL_FACE);

  glBegin (GL_QUADS);

  glColor3f (0.6, 0.6, 0.6);
  
  glNormal3f (0.0, 1.0, 0.0);
  glVertex3f (1.5, 0.0, 0.0);
  glVertex3f (1.5, 0.0, 7.0);
  glVertex3f (7.0, 0.0, 8.0);
  glVertex3f (7.0, 0.0, 4.0);

  glEnd ();

  glBegin (GL_TRIANGLES);

  glNormal3f (2.0, 4.0, 0.0);
  glVertex3f (7.0, 0.0, 4.0);
  glVertex3f (7.0, 0.0, 8.0);
  glVertex3f (8.0, -0.5, 8.0);

  glEnd ();

  glBegin (GL_QUADS);

  glNormal3f (0.0, 1.0, 0.0);
  glVertex3f (-1.5, 0.0, 0.0);
  glVertex3f (-7.0, 0.0, 4.0);
  glVertex3f (-7.0, 0.0, 8.0);
  glVertex3f (-1.5, 0.0, 7.0);

  glEnd ();

  glBegin (GL_TRIANGLES);

  glNormal3f (-2.0, 4.0, 0.0);
  glVertex3f (-7.0, 0.0, 4.0);
  glVertex3f (-8.0, -0.5, 8.0);
  glVertex3f (-7.0, 0.0, 8.0);

  glEnd ();

  glBegin (GL_TRIANGLES);

  glNormal3f (-1.0, 0.0, 0.0);

  glVertex3f (0.0, 0.5, 4.0);
  glVertex3f (0.0, 3.0, 8.0);
  glVertex3f (0.0, 0.5, 7.0);

  glEnd ();

  glDisable (GL_LIGHTING);
  glColor3f (0.5, 0.5, 0.5);
  
  glBegin (GL_LINES);

  glVertex3f (7.0, 0.0, 4.0);
  glVertex3f (7.0, 0.0, -8.0);

  glVertex3f (-7.0, 0.0, 4.0);
  glVertex3f (-7.0, 0.0, -8.0);
  
  glEnd ();

  glEndList ();
}


/*
 * drawVessel
 *
 * Input:
 *    data
 *       Pointer to transfer data.
 *    eye
 *       Pointer to eyepoint data.
 *
 * Description:
 *    Draws a vessel based on the transferred data.
 *
 */

void
drawVessel (xferData            *data,
	    eyePointXferData    *eye)
{
  vesselXferData    vessel;

  memcpy ((char *) &vessel, (char *) &data->data, sizeof (vessel));
  
  glPushMatrix ();
  glTranslatef (vessel.position[0],
		vessel.position[1],
		vessel.position[2]);
  matchDOF (vessel.heading, vessel.up);
  glScalef (1.0, 1.0, -1.0); 

  glCallList (vesselDL);
  
  glPopMatrix ();
}


