/***************************************************************************
 *
 * s_userinput.c - Server side user input functions
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <assert.h>
#include <stdio.h>

#include <vector.h>
#include <communication.h>
#include <errno.h>

#include <general.h>
#include <transfer.h>
#include <server.h>
#include <s_userinput.h>
#include <s_world.h>
#include <s_simulation.h>


/*
 * readKey
 *
 * Input:
 *    client
 *       Pointer to client entry.
 *
 * Description:
 *    Reads user input from client datagram communication channel
 *    and updates the corresponding vessel control information.
 *
 */

void
readKey (clientConnectionData   *client)
{
  message       msg;
  float         percent;
  vesselData   *vessel;

  comRead (&(client->dgram), &msg, sizeof (msg));
  vessel = fetchVesselData (client);
  
  switch (msg.type)
    {
    case SD_MSG_THRUST:
      percent = *((float *) (&msg.data));
      vessel->thrust = percent;
      break;

    case SD_MSG_HEADING:
      percent = *((float *) (&msg.data));
      vessel->headingThrust = percent;
      break;

    case SD_MSG_PITCH:
      percent = *((float *) (&msg.data));
      vessel->pitchThrust = percent;
      break;

    case SD_MSG_ROLL:
      percent = *((float *) (&msg.data));
      vessel->rollThrust = percent;
      break;

    case SD_MSG_FIRE:
      ctrlFire (client->myData, msg.data);
      break;

    default:
      printf ("ERROR: unknown message type: %d\n", msg.type);
      exit (SD_ERROR);
      break;
    }
}



/*
 * readInput
 *
 * Description:
 *    Poll client datagram channels for user input.
 *
 */

void
readInput (void)
{
  clientConnectionData   *client;
  linkedlist             *l;
  int                     res;
  int                     hup;

  l = getClientList ();
  if (llGetLength (l) <= 0)
    {
      return;
    }

  llFirst (l);
  do
    {
      client = (clientConnectionData *) llGetData (l);

      hup = comPoll (&(client->stream));
      if (hup == COM_POLL_DATA)
	{
	  deleteClient (client);
	}
      else
	{
	  do
	    {
	      res = comPoll (&(client->dgram));
	      
	      switch (res)
		{
		case -1:
		  printf ("ERROR: comPoll error: %s\n", strerror (errno));
		  exit (SD_ERROR);
		  break;
		  
		case COM_POLL_EXCP:
		  deleteClient (client);
		  res = COM_POLL_NODATA;   /* To break the while loop */
		  break;
		  
		case COM_POLL_DATA:
		  readKey (client);	      
		  break;
		  
		default:
		  break;
		}
	    } while (res != COM_POLL_NODATA);
	}
    }
  while (llNext (l));
}
