/***************************************************************************
 *
 * timeutils.c - Time measurning utilities.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <sys/time.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <timeutils.h>

/*
 * timeSleepUsec
 *
 * Input:
 *    usec
 *       Number of microseconds.
 *
 * Description:
 *    Puts the current process to sleep for the specified number of
 *    microseconds.
 *
 */

void
timeSleepUsec (int usec)
{
  struct timeval     timeout;

  timeout.tv_sec = 0;
  timeout.tv_usec = usec;

  select (0, NULL, NULL, NULL, &timeout);
}


/*
 * timeSleepMsec
 *
 * Input:
 *    msec
 *       Number of milliseconds.
 *
 * Description:
 *    Puts the current process to sleep for the specified number of
 *    milliseconds.
 *
 */

void
timeSleepMsec (int msec)
{
  struct timeval     timeout;

  timeout.tv_sec = 0;
  timeout.tv_usec = msec * TIME_MSEC_TO_USEC;

  select (0, NULL, NULL, NULL, &timeout);
}


/*
 * getTimeStamp
 *
 * Returns:
 *    A time stamp.
 *
 */

timestamp 
getTimestamp (void)
{
  struct timeval   t;
  double           usecs;

  if (gettimeofday (&t, NULL) == -1)
    {
      printf ("ERROR: couldn't gettimeofday: %s\n", strerror (errno));
      exit (-1);
    }

  usecs = ((double) t.tv_sec * TIME_SEC_TO_USEC) + (double) t.tv_usec;
  return usecs;
}


/*
 * timeDifference
 *
 * Input:
 *   t
 *      Time stamp.
 *
 * Returns:
 *    Number of seconds between t and now.
 *
 */

double
timeDifference (timestamp   t)
{
  struct timeval   tv;
  double           usecs;

  if (gettimeofday (&tv, NULL) == -1)
    {
      printf ("ERROR: couldn't gettimeofday: %s\n", strerror (errno));
      exit (-1);
    }

  usecs = ((double) tv.tv_sec * TIME_SEC_TO_USEC) + (double) tv.tv_usec;

  return (usecs - t) * TIME_USEC_TO_SEC;
}
