/***************************************************************************
 *
 * transfer.c - Data exchanged by server and client.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <string.h>

#include <general.h>
#include <communication.h>

#include <transfer.h>



char handshakeString[SD_HANDSHAKESIZE] = "SpaceDuel3D - PROTOTYPE VERSION";


/*
 * sendHandshake
 *
 * Input:
 *    channel
 *       Communication channel.
 *
 * Description:
 *    Sends the local handshake string through the channel.
 *
 * Returns:
 *    SD_OK if successful.
 *    SD_ERROR if unsuccessful.
 *
 */

int
sendHandshake (comChannel    *channel)
{
  if (comWrite (channel, handshakeString, SD_HANDSHAKESIZE) < 0)
    {
      return SD_ERROR;
    }

  return SD_OK;
}


/*
 * readHandshake
 *
 * Input:
 *    channel
 *       Communication channel.
 *
 * Description:
 *    Reads the handshake string from the channel and compares it
 *    to the local handshake string.
 *
 * Returns:
 *    SD_OK if successful and the strings match.
 *    SD_ERROR if unsuccessful or the strings doesn't match.
 *
 */

int
readHandshake (comChannel    *channel)
{
  char    str[SD_HANDSHAKESIZE];

  if (comRead (channel, str, SD_HANDSHAKESIZE) < 0)
    {
      return SD_ERROR;
    }

  if (strcmp (str, handshakeString) != 0)
    {
      return SD_ERROR;
    }

  return SD_OK;
}
