/***************************************************************************
 *
 * c_blast.c - Client-side functions for drawing a blast.
 *
 ***************************************************************************
 *
 * SpaceDuel3D, version 1.0.
 * Copyright (C) 1998 Gustav Taxen.
 * This is free software with ABSOLUTELY NO WARRANTY.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 ***************************************************************************/

#include <GL/glut.h>
#include <string.h>
#include <math.h>

#include <client.h>
#include <c_blast.h>

GLint     blastDL = 0;      /* holds name of blast display list */


/*
 * createBlastDL
 *
 * Description:
 *    Creates the blast display list.
 *
 */

void
createBlastDL (void)
{
  blastDL = glGenLists (1);

  glNewList (blastDL, GL_COMPILE);

  glShadeModel (GL_SMOOTH);

  glScalef (4.0, 4.0, 4.0);

  glBegin (GL_TRIANGLE_FAN);
  glColor3f (1.0, 1.0, 1.0);
  /*  glNormal3f (0.0, 0.0, 1.0);*/
  glVertex3f ( 0.0,  0.0,  0.0);

  /*  glNormal3f (0.0, 1.0, 0.0);*/
  glVertex3f ( 0.0,  0.1, -0.1);

  /*  glNormal3f (-1.0, 0.0, 0.0);*/
  glVertex3f (-0.1,  0.0, -0.1);

  /*  glNormal3f (0.0, -1.0, 0.0);*/
  glVertex3f ( 0.0, -0.1, -0.1);

  /*  glNormal3f (1.0, 0.0, 0.0);*/
  glVertex3f ( 0.1,  0.0, -0.1);

  /*  glNormal3f (0.0, 1.0, 0.0);*/
  glVertex3f ( 0.0,  0.1, -0.1);
  glEnd ();

  glBegin (GL_TRIANGLE_FAN);
  glColor3f (1.0, 1.0, 0.0);
  /*  glNormal3f (0.0, 0.0, -1.0);*/
  glVertex3f ( 0.0,  0.0, -1.0);
  
  glColor3f (1.0, 1.0, 1.0);
  /*  glNormal3f (0.0, 1.0, 0.0);*/
  glVertex3f ( 0.0,  0.1, -0.1);

  /*  glNormal3f (1.0, 0.0, 0.0);*/
  glVertex3f ( 0.1,  0.0, -0.1);

  /*  glNormal3f (0.0, -1.0, 0.0);*/
  glVertex3f ( 0.0, -0.1, -0.1);

  /*  glNormal3f (-1.0, 0.0, 0.0);*/
  glVertex3f (-0.1,  0.0, -0.1);
  
  /*  glNormal3f (0.0, 1.0, 0.0);*/
  glVertex3f ( 0.0,  0.1, -0.1);
  glEnd ();

  glEndList ();
}


/*
 * drawBlast
 *
 * Input:
 *    data
 *       Pointer to transfer data.
 *
 * Description:
 *    Draws a blast based on the transferred data.
 *
 */

void 
drawBlast (xferData           *data,
	   eyePointXferData   *eye)
{
  blastXferData    blast;

  memcpy ((char *) &blast, (char *) &data->data, sizeof (blast));

  if (nearEnough (blast.position, eye))
    {
      if (pointVisible (blast.position, eye))
	{
	  glPushMatrix ();

	  /* Transform to blast position and rotate to fit heading */

	  glTranslatef (blast.position[0],
			blast.position[1],
			blast.position[2]);
	  matchDOF (blast.heading, blast.up);

	  glEnable (GL_DEPTH_TEST);
	  glCallList (blastDL);

	  glPopMatrix ();
	}
    }
}
